%% BEGIN TESTSCRIPT

%% Options:
%%
%%   1. Add main-dir to path
%%   2. Use relative or full path for main-dir
%%   3. If main-dir was added, remove it after function handles
%%      defined?
%%   4. Clear functions after defining function handles?
%%
%% Valid combinations:
%%
%%   1 is T and: 2 is T/F, 3 is T/F, 4 is T/F (8 possibilities)
%%   2 is F and: 2,3 are irrelevant, 4 is T/F (2 possibilities)

clear functions

topdir = pwd ();

rel_main_dir = 'main-dir'
main_dir = fullfile (topdir, 'main-dir');
other_dir = fullfile (topdir, 'other-dir');
third_dir = fullfile (topdir, 'third-dir');

if (add_main_dir_to_path)
  main_dir_added_to_path = true;
  if (use_relative_main_dir)
    addpath (rel_main_dir);
    %% show it
    path
  else
    addpath (main_dir);
  end
else
  main_dir_added_to_path = false;
  cd (main_dir);
end

%% Define function handles in main-dir (or expecting them to come from
%% main-dir if it is first in the path but not the current directory).

a = 3/2;
fh_anon = @(x) fprintf ('fh_anon: %f\n', sin (a*x));
fh_simple = @simplefun;
fh_self = selffun ();
fh_missing = @nonexistent_fun;
[fh_priv, fh_sub, fh_set, fh_get] = parentfun ();
fh_pkg = @mypkg.pkgfun;
x = myclass ();
fh_lcl = myclass.get_lcl_hdl ();
fh_stat_meth = @myclass.stat_meth;
fh_meth = x.get_meth_hdl ();

%% Use the handles to call the functions.  Omit fh_missing here since
%% it is not defined in main-dir.

fh_anon (pi);
fh_simple ();
fh_self ();
fh_priv ();
fh_sub ();
fh_set (13);
fh_get ();
fh_pkg ();
fh_lcl ();
fh_stat_meth ();
fh_meth (x);

%% Examine handle info.

functions (fh_anon)
functions (fh_simple)
functions (fh_self)
functions (fh_missing)
functions (fh_priv)
functions (fh_sub)
functions (fh_set)
functions (fh_get)
functions (fh_pkg)
functions (fh_lcl)
functions (fh_stat_meth)
functions (fh_meth)

if (clear_functions_after_main_dir)
  clear functions
end

if (main_dir_added_to_path)
  if (remove_main_dir_from_path)
    rmpath (main_dir);
  end
end

cd (other_dir)

%% Use the handles to call the functions.  Include fh_missing here
%% since it is defined in other-dir.  There are also other versions of
%% functions with the same names as the simple function, local
%% function, etc.  Are they called, or those handles still bound to
%% their original definitions (that's what I would expect)?

fh_anon (pi);
fh_simple ();
fh_self ();
fh_missing ();
fh_priv ();
fh_sub ();
fh_set (13);
fh_get ();
fh_pkg ();
fh_lcl ();
fh_stat_meth ();
fh_meth (x);

%% Examine handle info after moving to other-dir, possibly after
%% removing main-dir from path and clearing functions.

functions (fh_anon)
functions (fh_simple)
functions (fh_self)
functions (fh_missing)
functions (fh_priv)
functions (fh_sub)
functions (fh_set)
functions (fh_get)
functions (fh_pkg)
functions (fh_lcl)
functions (fh_stat_meth)
functions (fh_meth)

cd (third_dir)

%% Use the handles to call the functions.  Include fh_missing here
%% since it is defined in third-dir.  There are also other versions of
%% functions with the same names as the simple function, local
%% function, etc.  Are they called, or those handles still bound to
%% their original definitions (that's what I would expect)?

fh_anon (pi);
fh_simple ();
fh_self ();
fh_missing ();
fh_priv ();
fh_sub ();
fh_set (13);
fh_get ();
fh_pkg ();
fh_lcl ();
fh_stat_meth ();
fh_meth (x);

%% Examine handle info after moving to third-dir, possibly after
%% removing main-dir from path and clearing functions.

functions (fh_anon)
functions (fh_simple)
functions (fh_self)
functions (fh_missing)
functions (fh_priv)
functions (fh_sub)
functions (fh_set)
functions (fh_get)
functions (fh_pkg)
functions (fh_lcl)
functions (fh_stat_meth)
functions (fh_meth)

cd (top_dir)

%% Use the handles to call the functions.  Omit fh_missing here
%% since it is not defined in the top directory.

fh_anon (pi);
fh_simple ();
fh_self ();
fh_priv ();
fh_sub ();
fh_set (13);
fh_get ();
fh_pkg ();
fh_lcl ();
fh_stat_meth ();
fh_meth (x);

%% Examine handle info after moving to the top directory, possibly
%% after removing main-dir from path and clearing functions.

functions (fh_anon)
functions (fh_simple)
functions (fh_self)
functions (fh_missing)
functions (fh_priv)
functions (fh_sub)
functions (fh_set)
functions (fh_get)
functions (fh_pkg)
functions (fh_lcl)
functions (fh_stat_meth)
functions (fh_meth)

%% END TESTSCRIPT
